// DmWriteBug.C
//
// Copyright 1997 by Alan Jay Weiner
// All Rights Reserved.
//----------------------------------------------------------------------


#define AJWBugsCreatorID		'AJWX'		// NOTE:  NOT REGISTERED CREATOR ID!!



#include <Pilot.h>						// all the system toolbox headers
#include "DmWriteBugRsrc.h"		// resource definitions


// === prototypes ===

void SetDatabaseBackup(LocalID dbID);
DmOpenRef OpenDatabaseWritable(char * dbName);


// === global data ===

DWord a7start;



// set a database attributes for backup
//
void
SetDatabaseBackup(LocalID dbID)
{
	Err		error;
	UInt		attributes;
		
	error = DmDatabaseInfo(0, dbID, 0,&attributes,0,0,0,0,0,0,0,0,0);
	if (!error) {
		attributes |= dmHdrAttrBackup;
		error = DmSetDatabaseInfo(0, dbID, 0,&attributes,0,0,0,0,0,0,0,0,0);
	}
}


// open a maze for reading and writing
//
DmOpenRef
OpenDatabaseWritable(char * dbName)
{
	LocalID				dbID;
	DmOpenRef			dbRef		= 0;


	dbID = DmFindDatabase(0, dbName);
	if (dbID) {
		SetDatabaseBackup(dbID);					// set so database will be backed up
		
		dbRef = DmOpenDatabase(0, dbID, dmModeReadWrite);
	}
	
	return dbRef;
}


static asm DWord GetA7(void)
{
	move.l	a7, d0
	sub.l		#4, d0		// adjust for call to GetA7()  (return addr)
	rts
}



static void AlertA7Value(char * pStr, DWord a7val)
{
	char	buf[100];
	char	buf1[10];
	

	StrCopy(buf, ",\nA7    = ");
	
	StrIToH(buf1, a7val);
	StrCat(buf, buf1);
	
	StrCat(buf,   "\ndepth = ");
	StrIToH(buf1, a7start-a7val);
	StrCat(buf, buf1);

	
	FrmCustomAlert(SingleButtonInfoAlert, pStr, buf, " ");
}





/***********************************************************************
 *
 * FUNCTION:		PilotMain
 *
 * DESCRIPTION:	This function is the equivalent of a main() function
 *						under standard C.  It is called by the Emulator to begin
 *						execution of this application.
 *
 * PARAMETERS:		cmd - command specifying how to launch the application.
 *						cmdPBP - parameter block for the command.
 *						launchFlags - flags used to configure the launch.			
 *
 * RETURNED:		Any applicable error codes.
 *
 ***********************************************************************/
 

 
DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
	DmOpenRef				dmDB;
	char *					dbName		= "junk";
	Err							error;
	UShort					recnum			= 0;
	VoidHand				hRecord;
	void *					pRec;
	Word						i;
	DWord						a7val;
	
	
	//  Check for a normal launch.
	if (cmd == sysAppLaunchCmdNormalLaunch) {


		a7val = GetA7();
		a7start = a7val;														// set this as the baseline
		AlertA7Value("At start of program", a7val);

		// create a maze database
		error = DmCreateDatabase(0, dbName, AJWBugsCreatorID, 'Data', false);
		if (error) {
			FrmCustomAlert(SingleButtonErrorAlert,
							"Error creating new database \"", dbName, "\"\nDeleting and recreating ");
			error = DmDeleteDatabase(0, DmFindDatabase(0, dbName));
			if (error) {
				FrmCustomAlert(SingleButtonErrorAlert,
							"Error deleting old database \"", dbName, "\"\nbailing out...");
				goto exit;
			}
			
			// recreate database
			
			error = DmCreateDatabase(0, dbName, AJWBugsCreatorID, 'Data', false);
			if (error) {
				FrmCustomAlert(SingleButtonErrorAlert,
							"SECOND Error creating new database \"", dbName, "\"\nbailing out...");
				goto exit;
			}
		}


		dmDB = OpenDatabaseWritable(dbName);
		if (0 == dmDB) {
			FrmCustomAlert(SingleButtonErrorAlert,
						"Error opening new database \"", dbName, "\"\nbailing out...");
			goto exit;
		}
		
		a7val = GetA7();
		AlertA7Value("After opening the database", a7val);

				
		// create a new record (20-byte record at index 0)
		
		hRecord = DmNewRecord(dmDB, 0, 20);

		
		// get a pointer for that record
		
		pRec = MemHandleLock(hRecord);
		
				
		// repeatably write to the first word of the record


		a7val = GetA7();
		AlertA7Value("before 1st call to DmWrite", a7val);
		
		DmWrite(pRec, 0, &i, 2);
			
		a7val = GetA7();
		AlertA7Value("between 1st and 2nd call to DmWrite", a7val);

		DmWrite(pRec, 0, &i, 2);
			
		a7val = GetA7();
		AlertA7Value("between 2nd and 3rd call to DmWrite", a7val);

		DmWrite(pRec, 0, &i, 2);
			
		a7val = GetA7();
		AlertA7Value("after 3rd call to DmWrite and before entering for-loop", a7val);


		// compare the difference with this loop!
		
		for (i=0; i<3; i++) {
			a7val = GetA7();
			AlertA7Value("in loop - before DmWrite", a7val);

			DmWrite(pRec, 0, &i, 2);

			a7val = GetA7();
			AlertA7Value("in loop - after  DmWrite", a7val);
		}


		a7val = GetA7();
		AlertA7Value("after loop, before releasing record ", a7val);

		// release the record
		
		MemHandleUnlock(hRecord);
		
		a7val = GetA7();
		AlertA7Value("after MemHandleUnlock() ", a7val);

		
		DmReleaseRecord(dmDB, recnum, true);
		
		a7val = GetA7();
		AlertA7Value("after DmReleaseRecord() ", a7val);

		

		// close the database
	
		error = DmCloseDatabase(dmDB);
		
		a7val = GetA7();
		AlertA7Value("after DmCloseDatabase() - before conditional", a7val);
		
		if (error) {
			FrmCustomAlert(SingleButtonErrorAlert,
						"Error closing database", " ", " ");
		}

		a7val = GetA7();
		AlertA7Value("after DmCloseDatabase() - after conditional", a7val);
		
exit:		
		// Pilot 1.0 will continuously relaunch this app unless we switch to 
		// another safe one.  The sysFileCDefaultApp is considered "safe".
		{
			Err	err;
			
			AppLaunchWithCommand(sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL);
		}

		a7val = GetA7();
		AlertA7Value("after switching apps; before return", a7val);
		
		return sysErrNotAllowed;
	}
	

	return 0;
}

